# Shenzen Wearable Technologies X9 Cordova Plugin

This plugin uses the [Bluetooth Low Energy (BLE) Central Plugin](https://github.com/don/cordova-plugin-ble-central) to connect to the SWT X9 wristband and return heart rate data and sleep data, and [katzer's Cordova Background Mode Plugin](https://github.com/katzer/cordova-plugin-background-mode) to do it in the background.

## Methods

- [scan](#scan)
- [readHeartRateData](#readHeartRateData)
- [readSleepData](#readSleepData)
- [getVerbosity](#getVerbosity)
- [toggleVerbosity](#toggleVerbosity)

## startScan

Starts a scan for X9 devices and returns the MAC address of the first X9 found.

### Example

```js
  X9.scan()
    .then((x9) => {
      console.log('X9 found! MAC address', x9);
    }),
    .catch((errorMessage) => {
      console.log(errorMessage);
    });
```

## readHeartRateData

Return heart rate data from an X9 device.  If no device MAC address is input and no X9 is connected, this method will scan for and (if found) connect to an X9, start the heart rate monitoring service, and return the response. It will take ~20 or more seconds to run the heart rate monitoring service and return the results.

### Example

```js
  X9.getHeartRateData() // can also supply specific MAC address as in: X9.getHeartRateData('00:11:22:33:FF:EE')
    .then((responseUintArray, responseStr) => {
      console.log(responseUintArray);
      console.log(responseStr);
    }),
    .catch((errorMessage) => {
      console.log(errorMessage);
    });
```

### Return Data

The ble plugin uses typed Arrays or ArrayBuffers for sending and receiving data. The success callback will return the data in the following format:

```js
  var responseUintArray = new Uint8Array(response);
  var responseStr = String.fromCharCode.apply(String, new Uint8Array(response));
  success(responseUintArray, responseStr);
```

## readSleepData

Return sleep data from an X9 device.  If no device MAC address is input and no X9 is connected, this method will scan for and (if found) connect to an X9 and return the response from the sleep data service. 

### Example

```js
  X9.getSleepData() // can also supply specific MAC address as in: X9.getSleepData('00:11:22:33:FF:EE')
    .then((responseUintArray, responseStr) => {
      console.log(responseUintArray);
      console.log(responseStr);
    }),
    .catch((errorMessage) => {
      console.log(errorMessage);
    });
```

### Return Data

The ble plugin uses typed Arrays or ArrayBuffers for sending and receiving data. The success callback will return the data in the following format:

```js
  var responseUintArray = new Uint8Array(response);
  var responseStr = String.fromCharCode.apply(String, new Uint8Array(response));
```

## isConnected

Checks if connected to an X9.

### Example

```js
  X9.isConnected() // Checks if connected to any previously-scanned X9
    .then((x9) => {
      console.log('Connected to X9! MAC address', x9);
    }),
    .catch((errorMessage) => {
      console.log(errorMessage);
    });

  X9.isConnected('35:6B:D8:F5:5E:DA') // Check if connected to a specific X9
    .then((x9) => {
      console.log('Connected to X9! MAC address', x9);
    }),
    .catch((errorMessage) => {
      console.log(errorMessage);
    });
  
  X9.isConnected(['35:6B:D8:F5:5E:DA']) // Check if connected to a specific X9 in addition to any previously-scanned X9
    .then((x9) => {
      console.log('Connected to X9! MAC address', x9);
    }),
    .catch((errorMessage) => {
      console.log(errorMessage);
    });
```

## disconnect

Disconnect from

### Example

```js
  X9.disconnect() // Disconnect from all devices
    .then((x9) => {
      console.log('Disconnected from X9. MAC address', x9);
    }),
    .catch((errorMessage) => {
      console.log(errorMessage);
    });

  X9.disconnect('35:6B:D8:F5:5E:DA') // Disconnect from specific X9
    .then((x9) => {
      console.log('Disconnected from X9. MAC address', x9);
    }),
    .catch((errorMessage) => {
      console.log(errorMessage);
    });
```

## getVerbosity

Return the plugin's debug (logging) mode: `false` (default) is for silent operation (no console logs,) `true` is for verbose logging.

### Example

```js
  X9.getVerbosity();
```

## toggleVerbosity

Toggle the plugin's debug (logging) mode: by default; no console logs are emitted; call `toggleVerbosity` to toggle verbose logging.

### Example

```js
  X9.toggleVerbosity();
```
