var SERVICE_USER_DATA = '1817';
var CHAR_START_HEART_RATE_MONITORING = '2a5f';
var CHAR_READ_HEART_RATE_DATA = '6e400007-b5a3-f393-e0a9-e50e24dcca9e';
var CHAR_READ_SLEEP_DATA = '6e400006-b5a3-f393-e0a9-e50e24dcca9e';

var START_HEART_RATE_MONITORING = 0x31;

var verbose = false;
var scanning = false;
var SCANNED_X9S = [];
var CONNECTED_X9_MAC;

// Perform one scan
// Returns a single promise
exports.scan = function() {
  return new Promise(function(resolve, reject) {
    if (verbose)
      console.log('cordova-plugin-swt-x9: scan started');
    scanning = true;

    ble.startScan([], function(device) {
      if (device['name'] == 'X9') {
        if (verbose)
          console.log('cordova-plugin-swt-x9: device found', JSON.stringify(device));
        if (SCANNED_X9S.indexOf(device['id']) < 0) { // New device found
          SCANNED_X9S.push(device['id']);
          ble.stopScan();
          scanning = false;
          resolve(device['id']);
        }
      }
    }, function(error) {
      if (verbose)
        console.error('cordova-plugin-swt-x9: scan error', error);
      scanning = false;
      reject(error);
    });
  });
}

exports.stopScan = function() {
  return new Promise(function(resolve, reject) {
    ble.stopScan(function(result) {
      if (verbose)
        console.log('cordova-plugin-swt-x9: stopped scan');
      resolve(result);
    }, function(error) {
      if (verbose)
        console.error('cordova-plugin-swt-x9: stopScan error');
      reject(error);
    });
  });
}

exports.connect = function(MAC = null) {
  return new Promise(function(resolve, reject) {
    let X9S = SCANNED_X9S;
    if (MAC) { // MAC address specified
      if (typeof MAC == 'string') {
        ble.connect(MAC, function() {
          // Connected to X9.
          if (verbose)
            console.log('cordova-plugin-swt-x9: connected', MAC);
          resolve(MAC);
        }, function() {
          if (verbose)
            console.log('cordova-plugin-swt-x9: not connected', MAC);
          reject();
        });
      } else {
        X9S = X9S.concat(MAC);

        return connectX9s(X9S)
          .then((result) => { resolve(result); })
          .catch((error) => { reject(error); });
      }
    } else {
      // Try each X9 in X9S, which is either just SCANNED_X9S or also the MACs specified in the MAC array parameter
      return connectX9s(X9S)
        .then((result) => { resolve(result); })
        .catch((error) => { reject(error); });
    }
  });
}

function connectX9s(X9S_TO_CONNECT = null) {
  return new Promise(function(resolve, reject) {
    if (!X9S_TO_CONNECT) {
      X9S_TO_CONNECT = SCANNED_X9S;
    }
    let X9_TO_CONNECT = X9S_TO_CONNECT.shift();
    ble.connect(X9_TO_CONNECT, function() {
      // Connected to X9.
      if (verbose)
        console.log('cordova-plugin-swt-x9: connected', X9_TO_CONNECT);
      resolve(X9_TO_CONNECT);
    }, function() {
      if (verbose)
        console.log('cordova-plugin-swt-x9: not connected', X9_TO_CONNECT);
      if (X9S_TO_CONNECT.length > 0) {
        return connectX9s(X9S_TO_CONNECT)
          .then((result) => { resolve(result); })
          .catch((error) => { reject(error); });
      } else {
        reject();
      }
    });
  });
}

exports.isConnected = function(MAC = null) {
  return new Promise(function(resolve, reject) {
    let X9S = SCANNED_X9S;
    if (MAC) { // MAC address specified
      if (typeof MAC == 'string') {
        ble.isConnected(MAC, function() {
          // Connected to X9.
          if (verbose)
            console.log('cordova-plugin-swt-x9: connected', MAC);
          resolve(MAC);
        }, function() {
          if (verbose)
            console.log('cordova-plugin-swt-x9: not connected', MAC);
          reject();
        });
      } else {
        X9S = X9S.concat(MAC);

        // Try each X9 in X9S, which is either just SCANNED_X9S or also the MACs specified in the MAC array parameter
        for (let X9 in X9S) {
          ble.isConnected(X9, function() {
            // Connected to X9.
            if (verbose)
              console.log('cordova-plugin-swt-x9: connected', X9S[X9]);
            resolve(X9S[X9]);
          }, function() {
            if (verbose)
              console.log('cordova-plugin-swt-x9: not connected', X9S[X9]);
            let index = X9S.indexOf(X9S[X9]);
            if (index > -1) {
              X9S.splice(index, 1);
            }
            if (X9S.length <= 0) {
              reject();
            }
          });
        }
      }
    } else {
      // Try each X9 in X9S, which is either just SCANNED_X9S or also the MACs specified in the MAC array parameter
      for (let X9 in X9S) {
        ble.isConnected(X9, function() {
          // Connected to X9.
          if (verbose)
            console.log('cordova-plugin-swt-x9: connected', X9S[X9]);
          resolve(X9S[X9]);
        }, function() {
          if (verbose)
            console.log('cordova-plugin-swt-x9: not connected', X9S[X9]);
          let index = X9S.indexOf(X9S[X9]);
          if (index > -1) {
            X9S.splice(index, 1);
          }
          if (X9S.length <= 0) {
            reject();
          }
        });
      }
    }
  });
}

exports.disconnect = function(MAC = null) {
  return new Promise(function(resolve, reject) {
    if (MAC) { // MAC address specified
      ble.disconnect(MAC, function() {
        // Disconnected from X9
        if (verbose)
          console.log('cordova-plugin-swt-x9: disconnected', MAC);
        resolve(MAC);
      }, function() {
        if (verbose)
          console.log('cordova-plugin-swt-x9: error disconnecting', MAC);
        reject();
      });
    } else { // No MAC address specific; check if connected
      if (CONNECTED_X9_MAC) { // Connected to X9; disconnect from it
        ble.disconnect(CONNECTED_X9_MAC, function() {
          // Disconnected from X9
          if (verbose)
            console.log('cordova-plugin-swt-x9: disconnected', CONNECTED_X9_MAC);
          resolve(CONNECTED_X9_MAC);
        }, function() {
          if (verbose)
            console.log('cordova-plugin-swt-x9: error disconnecting', CONNECTED_X9_MAC);
          reject();
        });
      } else { // No specific X9 connected; check all previously-scanned X9s, disconnect if connected
        // Try each X9 in SCANNED_X9S
        for (let X9 in SCANNED_X9S) {
          ble.isConnected(X9, function() {
            // Connected to X9.
            if (verbose)
              console.log('cordova-plugin-swt-x9: connected', X9);
            ble.disconnect(X9, function() {
              // Disconnected from X9
              if (verbose)
                console.log('cordova-plugin-swt-x9: disconnected', X9);
              resolve(X9);
            }, function() {
              if (verbose)
                console.log('cordova-plugin-swt-x9: error disconnecting', X9);
              reject();
            });
          }, function() {
            if (verbose)
              console.log('cordova-plugin-swt-x9: not connected', X9);
            // Do not reject so as to check each X9
          });
        }
        // Not connected to any known X9s
        reject();
      }
    }
  });
}

// Get X9 heart rate data
// If not connected and if no specific MAC found, it will start a scan, connect, etc.
exports.readHeartRateData = function(MAC) {
  return new Promise(function(resolve, reject) {
    if (!MAC)
      MAC = CONNECTED_X9_MAC
    
    if (!MAC && !CONNECTED_X9_MAC) {
      // No MAC provided, no connected X9 MAC found, need to start scan, connect, etc...
      // TODO
    } else {
      ble.isConnected(MAC, function() {
        // Connected to X9.
        if (verbose)
          console.log('cordova-plugin-swt-x9: writing to start heart rate reading process (1)');
        var dataStartHeartMonitoring = new Uint8Array(1);
        dataStartHeartMonitoring[0] = START_HEART_RATE_MONITORING;
        ble.write(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_START_HEART_RATE_MONITORING, dataStartHeartMonitoring.buffer, function(response1) {
          if (verbose)
            console.log('cordova-plugin-swt-x9: started heart rate service, listening for notification response (1)');
          ble.startNotification(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_READ_HEART_RATE_DATA, function(response2) {
            if (verbose)
              console.log('cordova-plugin-swt-x9: heart data notification response, stopping notifications... (1)');
            var responseUintArray = new Uint8Array(response2);
            var responseStr = String.fromCharCode.apply(String, new Uint8Array(response2));
            resolve(responseUintArray, responseStr);
          }, function(error) {
            if (verbose)
              console.error('cordova-plugin-swt-x9: heart rate notification error (1)');
            reject(error);
          });
        }, function(error) {
          if (verbose)
            console.error('cordova-plugin-swt-x9: write error (1)');
          reject(error);
        });
      }, function() {
        // X9 not connected, need to connect...
        if (verbose)
          console.log('cordova-plugin-swt-x9: connecting to X9...');
        if (SCANNED_X9S.indexOf(MAC) > -1) {
          // Specified X9 MAC has been previously scanned, connect
          ble.connect(MAC, function(device) {
            if (verbose)
              console.log('cordova-plugin-swt-x9: connected to X9');
            if (!CONNECTED_X9_MAC || CONNECTED_X9_MAC != MAC) {
              CONNECTED_X9_MAC = MAC;
            }

            if (verbose)
              console.log('cordova-plugin-swt-x9: writing to start heart rate reading process (2)');
            var dataStartHeartMonitoring = new Uint8Array(1);
            dataStartHeartMonitoring[0] = START_HEART_RATE_MONITORING;
            ble.write(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_START_HEART_RATE_MONITORING, dataStartHeartMonitoring.buffer, function(response1) {
              if (verbose)
                console.log('cordova-plugin-swt-x9: started heart rate service, listening for notification response (2)');
              ble.startNotification(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_READ_HEART_RATE_DATA, function(response2) {
                if (verbose)
                  console.log('cordova-plugin-swt-x9: heart data notification response, stopping notifications...');
                var responseUintArray = new Uint8Array(response2);
                var responseStr = String.fromCharCode.apply(String, new Uint8Array(response2));
                resolve(responseUintArray, responseStr);
              }, function(error) {
                if (verbose)
                  console.error('cordova-plugin-swt-x9: heart rate notification error (2)');
                reject(error);
              });
            }, function(error) {
              if (verbose)
                console.error('cordova-plugin-swt-x9: write error (2)');
              reject(error);
            });
          }, function(error) {
            if (verbose)
              console.error('cordova-plugin-swt-x9: connection error (2)');
            reject(error);
          });
        } else {
          // Specified X9 MAC has not been scanned previously, scan for it and connect if found
          ble.startScan([], function(device) {
            if (device['name'] == 'X9') {
              if (verbose)
                console.log('cordova-plugin-swt-x9: device found', JSON.stringify(device));
              if (device['id'] == MAC) { // X9 found...
                ble.connect(MAC, function(device) {
                  if (verbose)
                    console.log('cordova-plugin-swt-x9: connected to X9');
                  if (!CONNECTED_X9_MAC || CONNECTED_X9_MAC != MAC) {
                    CONNECTED_X9_MAC = MAC;
                  }

                  if (verbose)
                    console.log('cordova-plugin-swt-x9: writing to start heart rate reading process (3)');
                  var dataStartHeartMonitoring = new Uint8Array(1);
                  dataStartHeartMonitoring[0] = START_HEART_RATE_MONITORING;
                  ble.write(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_START_HEART_RATE_MONITORING, dataStartHeartMonitoring.buffer, function(response1) {
                    if (verbose)
                      console.log('cordova-plugin-swt-x9: started heart rate service, listening for notification response (3)');
                    ble.startNotification(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_READ_HEART_RATE_DATA, function(response2) {
                      if (verbose)
                        console.log('cordova-plugin-swt-x9: heart data notification response, stopping notifications...');
                      var responseUintArray = new Uint8Array(response2);
                      var responseStr = String.fromCharCode.apply(String, new Uint8Array(response2));
                      resolve(responseUintArray, responseStr);
                    }, function(error) {
                      if (verbose)
                        console.error('cordova-plugin-swt-x9: heart rate notification error (3)');
                      reject(error);
                    });
                  }, function(error) {
                    if (verbose)
                      console.error('cordova-plugin-swt-x9: write error (3)');
                    reject(error);
                  });
                }, function(error) {
                  if (verbose)
                    console.error('cordova-plugin-swt-x9: connection error (3)');
                  reject(error);
                });
              }
            }
          }, function(error) {
            if (verbose)
              console.error('cordova-plugin-swt-x9: scan error', error);
            scanning = false;
            reject(error);
          });

          // TODO: timeout
        }
      });
    }
  });
}

// Get X9 sleep data
// If not connected and if no specific MAC found, it will start a scan, connect, etc.
exports.readSleepData = function(MAC) {
  return new Promise(function(resolve, reject) {
    if (!MAC)
      MAC = CONNECTED_X9_MAC
    
    if (!MAC && !CONNECTED_X9_MAC) {
      // No MAC provided, no connected X9 MAC found, need to start scan, connect, etc...
      // TODO
    } else {
      ble.isConnected(MAC, function() {
        // Connected to X9.
        if (verbose)
          console.log('cordova-plugin-swt-x9: listening for sleep data notification response (1)');
        ble.startNotification(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_READ_SLEEP_DATA, function(response) {
          if (verbose)
            console.log('cordova-plugin-swt-x9: sleep data notification response (1)', response);
          var responseUintArray = new Uint8Array(response);
          var responseStr = String.fromCharCode.apply(String, new Uint8Array(response));
          resolve(responseUintArray, responseStr);
        }, function(error) {
          if (verbose)
            console.error('cordova-plugin-swt-x9: sleep notification error (1)');
          reject(error);
        });
      }, function() {
        // X9 not connected, need to connect...
        if (verbose)
          console.log('cordova-plugin-swt-x9: connecting to X9...');
        if (SCANNED_X9S.indexOf(MAC) > -1) {
          // Specified X9 MAC has been previously scanned, connect
          ble.connect(MAC, function(device) {
            if (verbose)
              console.log('cordova-plugin-swt-x9: connected to X9');
            if (!CONNECTED_X9_MAC || CONNECTED_X9_MAC != MAC) {
              CONNECTED_X9_MAC = MAC;
            }

            if (verbose)
              console.log('cordova-plugin-swt-x9: listening for sleep data notification response (2)');
            ble.startNotification(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_READ_SLEEP_DATA, function(response) {
              if (verbose)
                console.log('cordova-plugin-swt-x9: sleep data notification response (2)', response);
              var responseUintArray = new Uint8Array(response);
              var responseStr = String.fromCharCode.apply(String, new Uint8Array(response));
              resolve(responseUintArray, responseStr);
            }, function(error) {
              if (verbose)
                console.error('cordova-plugin-swt-x9: sleep notification error (2)');
              reject(error);
            });
          }, function(error) {
            if (verbose)
              console.error('cordova-plugin-swt-x9: connection error (2)');
            reject(error);
          });
        } else {
          // Specified X9 MAC has not been scanned previously, scan for it and connect if found
          ble.startScan([], function(device) {
            if (device['name'] == 'X9') {
              if (verbose)
                console.log('cordova-plugin-swt-x9: device found', JSON.stringify(device));
              if (device['id'] == MAC) { // X9 found...
                ble.connect(MAC, function(device) {
                  if (verbose)
                    console.log('cordova-plugin-swt-x9: connected to X9');
                  if (!CONNECTED_X9_MAC || CONNECTED_X9_MAC != MAC) {
                    CONNECTED_X9_MAC = MAC;
                  }

                  if (verbose)
                    console.log('cordova-plugin-swt-x9: listening for sleep data notification response (3)');
                  ble.startNotification(CONNECTED_X9_MAC, SERVICE_USER_DATA, CHAR_READ_SLEEP_DATA, function(response) {
                    if (verbose)
                      console.log('cordova-plugin-swt-x9: sleep data notification response (3)', response);
                    var responseUintArray = new Uint8Array(response);
                    var responseStr = String.fromCharCode.apply(String, new Uint8Array(response));
                    resolve(responseUintArray, responseStr);
                  }, function(error) {
                    if (verbose)
                      console.error('cordova-plugin-swt-x9: sleep notification error (3)');
                    reject(error);
                  });
                }, function(error) {
                  if (verbose)
                    console.error('cordova-plugin-swt-x9: connection error (3)');
                  reject(error);
                });
              }
            }
          }, function(error) {
            if (verbose)
              console.error('cordova-plugin-swt-x9: scan error', error);
            scanning = false;
            reject(error);
          });

          // TODO: timeout
        }
      });
    }
  });
}

exports.getVerbosity = function() {
  return verbose;
}

exports.toggleVerbosity = function() { // toggle verbose console logging mode
  verbose = !verbose;
  if (verbose)
    console.log('cordova-plugin-swt-x9: toggled debug mode, set to', verbose);
}
